@@ -27,34 +27,31 @@
 //! let mut exa_con = connect(&dsn, &schema, &user, &password).unwrap();
 //!
 //! // Executing a statement
-//! let result = exa_con.execute("SELECT '1', '2', '3' UNION ALL SELECT '4', '5', '6'").unwrap();
+//! let mut result = exa_con.execute("SELECT '1', '2', '3' UNION ALL SELECT '4', '5', '6'").unwrap();
 //!
-//! // The ResultSet is a lazy iterator, fetching rows as needed.
-//! // For that reason, it can only be iterated over when owned,
-//! // as it gets consumed in the process.
-//! if let QueryResult::ResultSet(r) = result {
-//!     // Each row is a Result<Vec<serde_json::Value>>
-//!     for row in r {
-//!         for col_val in row.unwrap() {
-//!             // Columns values are of type serde_json::Value
-//!             println!("{}", col_val);
-//!         }
-//!     }
-//! }
+//! // Retrieving data associated with the result set.
+//! // A Vec of rows is returned, and the row type in this case will be Vec<String>.
+//! let data = exa_con.iter_result(&mut result).collect::<Result<Vec<Vec<String>>>>().unwrap();
 //! ```
 //!
-//! Keeping in mind that the result set is an iterator when owned,
-//! the full iterator toolset can be used.
+//! The [Connection::iter_result] method returns a lazy iterator over a result's rows. Only a given
+//! rows buffer is present at a given time in a [ResultSet], which is contained in a [QueryResult],
+//! if the query had a result set. The buffer size can be changed either in the [ConOpts] or
+//! through the [Connection::set_fetch_size] method.
 //!
-//! Additionally, `TryFrom` is implemented for [QueryResult](crate::query::QueryResult) to make
-//! [ResultSet](crate::query::ResultSet) retrieval more convenient, instead of having to match
-//! against it (the same can be done for the `u32` representing the affected rows count).
+//! # Cleanup
+//! [QueryResult] structs will automatically close once fully retrieved. If that does not happen,
+//! they will behave like [PreparedStatement] structs in the sense that they should be manually
+//! closed, or they will be closed when the [Connection] is dropped.
+//!
+//! # Best practice
+//! As a best practice, you should always close [QueryResult] and [PreparedStatement] instances
+//! once they are no longer needed.
 //!
 //! ```
 //! use exasol::error::Result;
 //! use exasol::{connect, bind, QueryResult, ResultSet};
 //! use std::env;
-//! use serde_json::Value;
 //!
 //! let dsn = env::var("EXA_DSN").unwrap();
 //! let schema = env::var("EXA_SCHEMA").unwrap();
@@ -62,26 +59,29 @@
 //! let password = env::var("EXA_PASSWORD").unwrap();
 //!
 //! let mut exa_con = connect(&dsn, &schema, &user, &password).unwrap();
-//! let result = exa_con.execute("SELECT OBJECT_NAME FROM EXA_ALL_OBJECTS LIMIT 5000;").unwrap();
+//! let mut result = exa_con.execute("SELECT * FROM EXA_RUST_TEST LIMIT 1000;").unwrap();
+//! let mut counter = 0;
 //!
-//! ResultSet::try_from(result)
-//!     .unwrap()
-//!     .by_ref()
-//!     .take(500)
-//!     .collect::<Result<Vec<Vec<Value>>>>()
-//!     .unwrap()
-//!     .into_iter()
-//!     .map(|row| row[0].as_str().unwrap().to_owned())
-//!     .collect::<Vec<String>>();
+//! while counter < 3 {
+//!     // Only enough calls necessary to retrieve 100 rows will be made to the database.
+//!     // Any leftover data in the chunk will still be present in the ResultSet buffer.
+//!     // and can be used in later retrievals.
+//!     let data: Vec<(String, String, u16)> = exa_con.iter_result(&mut result).take(100).collect::<Result<_>>().unwrap();
+//!     // do stuff with data
+//!
+//!     counter += 1;
+//! }
+//!
+//! // Alternatively, you can retrieve row chunks while there still are rows in the result set.
+//! // while result.has_rows() {
+//!     // let data: Vec<(String, String, u16)> = exa_con.fetch(&mut result, 100).unwrap();
+//! // }
+//!
+//! // Failing to close the result here will leave it up to the
+//! // Connection to do it when it is dropped.
+//! exa_con.close_result(result);
 //! ```
 //!
-//! In the example above, note the `collect::<Result<Vec<Row>>>()` call.
-//! When iterating over a result set, because fetch requests may fail, every row
-//! (which by default is a [Vec<serde_json::Value>]) is in the form of a [Result].
-//!
-//! `collect` can handle that for us and convert a [Vec<Result<Row>>] into a [Result<Vec<Row>>],
-//! stopping on the first error encountered, if any.
-//!
 //! # Custom Row Type
 //!
 //! The crate implements a custom deserializer, which is a trimmed down specialized version
@@ -108,18 +108,19 @@
 //! let user = env::var("EXA_USER").unwrap();
 //! let password = env::var("EXA_PASSWORD").unwrap();
 //! let mut exa_con = connect(&dsn, &schema, &user, &password).unwrap();
-//! let result = exa_con.execute("SELECT 1, 2 UNION ALL SELECT 1, 2;").unwrap();
+//! let mut result = exa_con.execute("SELECT 1, 2 UNION ALL SELECT 1, 2;").unwrap();
 //!
 //! // Change the expected row type with the turbofish notation
-//! let mut result_set = ResultSet::try_from(result).unwrap().with_row_type::<(String, String)>();
-//! let row1 = result_set.next();
+//! let mut data: Vec<[u8; 2]> = exa_con.iter_result(&mut result).collect::<Result<_>>().unwrap();
+//! let row1 = data[0];
 //!
+//! // You can also rely on type inference.
 //! // Nothing stops you from changing row types
-//! // on the same result set, even while iterating through it
-//! let mut result_set = result_set.with_row_type::<Vec<String>>();
-//! let row2 = result_set.next();
+//! // on the same result set.
+//! let mut data: Vec<Vec<u8>> = exa_con.iter_result(&mut result).collect::<Result<_>>().unwrap();;
+//! let row2 = data.pop();
 //!
-//! let result = exa_con.execute("SELECT 1 as col1, 2 as col2, 3 as col3 \
+//! let mut result = exa_con.execute("SELECT 1 as col1, 2 as col2, 3 as col3 \
 //!                               UNION ALL \
 //!                               SELECT 4 as col1, 5 as col2, 6 as col3;").unwrap();
 //!
@@ -130,9 +131,8 @@
 //!     col3: u8,
 //! }
 //!
-//! let result_set = ResultSet::try_from(result).unwrap().with_row_type::<Test>();
-//! for row in result_set.with_row_type::<Test>() {
-//!     let ok_row = row.unwrap();
+//! let data = exa_con.iter_result::<Test>(&mut result).collect::<Result<Vec<_>>>().unwrap();
+//! for row in data {
 //!     // do stuff with row
 //! }
 //! ```
@@ -184,7 +184,7 @@
 //! The [Connection] struct can be directly instantiated with the use of [ConOpts].
 //!
 //! ```
-//! use exasol::{ConOpts, Connection};
+//! use exasol::*;
 //! use std::env;
 //!
 //! let dsn = env::var("EXA_DSN").unwrap();
@@ -195,18 +195,19 @@
 //! // Only providing fields for which we want custom values
 //! let mut opts = ConOpts::new();
 //! opts.set_dsn(dsn);
-//! opts.set_user(user);
-//! opts.set_password(password);
-//! opts.set_schema(schema);
+//! opts.set_login_kind(LoginKind::Credentials(Credentials::new(user, password)));
+//! opts.set_schema(Some(schema));
 //!
 //! let exa_con = Connection::new(opts).unwrap();
 //! ```
 //!
 //! # Features
 //!
-//! - `native-tls` - (disabled by default) enables `tungstenite` WSS support through native-tls
-//! - `rustls` - (disabled by default) enables `tungstenite` WSS support through rustls
-//! - `flate2` - (disabled by default) enables support for requests and responses compression
+//! - `native-tls` - (disabled by default) enables `tungstenite` WSS encryption support through native-tls
+//! - `native-tls-vendored` - (disabled by default) enables `tungstenite` WSS encryption support through native-tls-vendored
+//! - `rustls-tls-webpki-roots` - (disabled by default) enables `tungstenite` WSS encryption support through rustls-tls-webpki-roots
+//! - `rustls-tls-native-roots` - (disabled by default) enables `tungstenite` WSS encryption support through rustls-tls-native-roots
+//! - `flate2` - (disabled by default) enables compression support
 //!
 //! Enabling these features allows changing additional settings in [ConOpts] instances.
 //!
@@ -214,7 +215,7 @@
 //! Attempting to use these methods without their respective features enabled results in panics.
 //!
 //! ``` should_panic
-//! # use exasol::{ConOpts, Connection};
+//! # use exasol::*;
 //! # use std::env;
 //! #
 //! # let dsn = env::var("EXA_DSN").unwrap();
@@ -225,9 +226,8 @@
 //! let mut opts = ConOpts::new();
 //!
 //! opts.set_dsn(dsn);
-//! opts.set_user(user);
-//! opts.set_password(password);
-//! opts.set_schema(schema);
+//! opts.set_login_kind(LoginKind::Credentials(Credentials::new(user, password)));
+//! opts.set_schema(Some(schema));
 //!
 //! opts.set_encryption(true);
 //! opts.set_compression(true);
@@ -235,22 +235,23 @@
 //!
 //! # Batch Execution
 //!
-//! Faster query execution can be achieved through either
+//! Batch query execution can be achieved through either
 //! [Connection::execute_batch] or [PreparedStatement].
+//!
+//! # HTTP Transport
+//!
+//! Parallel, highly performant, IMPORT/EXPORT operations are supported through
+//! [Connection::export_to_closure] and [Connection::import_from_closure].
 
-extern crate core;
-
-pub mod con_opts;
-pub mod connection;
+mod con_opts;
+mod connection;
 pub mod error;
-pub mod params;
-pub mod query;
-pub mod response;
-pub mod row;
+mod params;
 
-pub use crate::con_opts::{ConOpts, ProtocolVersion};
-pub use crate::connection::{connect, Connection};
-pub use crate::params::bind;
-pub use crate::query::{PreparedStatement, QueryResult, ResultSet};
-pub use crate::response::{Column, DataType};
-pub use crate::row::deserialize_as_seq;
+pub use con_opts::{ConOpts, Credentials, LoginKind, ProtocolVersion};
+#[cfg(any(feature = "rustls", feature = "native-tls-basic"))]
+pub use connection::Connector;
+pub use connection::{connect, deserialize_as_seq, Connection};
+pub use connection::{Column, DataType, PreparedStatement, QueryResult, ResultSet};
+pub use connection::{ExportOpts, HttpTransportOpts, ImportOpts};
+pub use params::bind;
